@@ -32,12 +32,15 @@
 /* X and server generic header files */
 #include "xf86.h"
 #include "xf86_OSproc.h"
+#if GET_ABI_MAJOR(ABI_VIDEODRV_VERSION) < 6
 #include "xf86RAC.h"
 #include "xf86Resources.h"
+#endif
 #include "xf86cmap.h"
 #include "xf86xv.h"
 #include <xf86i2c.h>
 #include <xf86Crtc.h>
+#include <xf86RandR12.h>
 #include "vbe.h"
 
 /* PCI vendor/device definitions */
@@ -48,8 +51,6 @@
 #include "vgaHW.h"
 #include "dixstruct.h"
 
-/* mi banking wrapper */
-#include "mibank.h"
 /* initialise a SW cursor */
 #include "mipointer.h"
 /* implementing backing store */
@@ -58,10 +59,14 @@
 /* colormap initialization */
 #include "micmap.h"
 
-#include "xf86Version.h"
+#include "xorgVersion.h"
 
 #include "xgi_debug.h"
 
+//#define FB_ACCESS_WRAPPER
+#if X_BYTE_ORDER == X_BIG_ENDIAN
+#include "wfbrename.h"
+#endif
 #include "fb.h"
 
 /* Driver data structures */
@@ -86,6 +91,8 @@
 #include "xg47_cmdlist.h"
 #include "xg47_i2c.h"
 
+#include <byteswap.h>
+
 /* Jong 09/20/2006; support dual view */
 extern int		g_DualViewMode;
 
@@ -115,14 +122,14 @@ static Bool XGIPciProbe(DriverPtr drv, int entity_num, struct pci_device *dev,
 static Bool     XGIProbe(DriverPtr drv, int flags);
 #endif
 static Bool     XGIPreInit(ScrnInfoPtr pScrn, int flags);
-static Bool     XGIScreenInit(int Index, ScreenPtr pScreen, int argc, char **argv);
-static Bool     XGIEnterVT(int scrnIndex, int flags);
-static void     XGILeaveVT(int scrnIndex, int flags);
-static Bool     XGICloseScreen(int scrnIndex, ScreenPtr pScreen);
+static Bool     XGIScreenInit(SCREEN_INIT_ARGS_DECL);
+static Bool     XGIEnterVT(VT_FUNC_ARGS_DECL);
+static void     XGILeaveVT(VT_FUNC_ARGS_DECL);
+static Bool     XGICloseScreen(CLOSE_SCREEN_ARGS_DECL);
 static Bool     XGISaveScreen(ScreenPtr pScreen, int mode);
 /* Optional functions */
-static void     XGIFreeScreen(int scrnIndex, int flags);
-static int      XGIValidMode(int scrnIndex, DisplayModePtr mode, Bool verbose,
+static void     XGIFreeScreen(FREE_SCREEN_ARGS_DECL);
+static int      XGIValidMode(SCRN_ARG_TYPE arg, DisplayModePtr mode, Bool verbose,
                              int flags);
 
 /* Internally used functions */
@@ -136,7 +143,13 @@ static void     XGISave(ScrnInfoPtr pScrn);
 static void     XGIRestore(ScrnInfoPtr pScrn);
 static Bool     XGIModeInit(ScrnInfoPtr pScrn, DisplayModePtr mode);
 
-static void     XGIBlockHandler(int, pointer, pointer, pointer);
+static void     XGIBlockHandler(BLOCKHANDLER_ARGS_DECL);
+
+static void xg47_write_memory_func(void *dst, uint32_t value, int size);
+static uint32_t xg47_read_memory_func(const void *src, int size);
+static void xg47_setup_fb_wrap(ReadMemoryProcPtr *read_ptr, 
+    WriteMemoryProcPtr *write_ptr, DrawablePtr pDraw);
+static void xg47_finish_fb_wrap(DrawablePtr pDraw);
 
 
 static const char *vgahwSymbols[] = {
@@ -173,14 +186,19 @@ static const char *i2cSymbols[] = {
 };
 
 static const char *fbSymbols[] = {
+#if X_BYTE_ORDER == X_BIG_ENDIAN
+    "wfbPictureInit",
+    "wfbScreenInit",
+#else
     "fbPictureInit",
     "fbScreenInit",
+#endif
     NULL
 };
 
 static const char *xaaSymbols[] = {
     "XAACreateInfoRec",
-    "XAADestroyInfoRec",
+    "XAAestroyInfoRec",
     "XAAFillSolidRects",
     "XAAGetCopyROP",
     "XAAGetPatternROP", 
@@ -357,7 +375,7 @@ static XF86ModuleVersionInfo XGIVersionRec =
     MODULEVENDORSTRING,
     MODINFOSTRING1,
     MODINFOSTRING2,
-    XF86_VERSION_CURRENT,
+    XORG_VERSION_CURRENT,
     XGI_VERSION_MAJOR, XGI_VERSION_MINOR, XGI_VERSION_PATCH,
     ABI_CLASS_VIDEODRV,
     ABI_VIDEODRV_VERSION,
@@ -396,7 +414,13 @@ static pointer XGISetup(pointer module,
         isInited = TRUE;
         xf86AddDriver(&XGI, module, 1);
 
-        if (!LoadSubModule(module, "fb", NULL, NULL, NULL, NULL,
+        if (!LoadSubModule(module, 
+#if X_BYTE_ORDER == X_BIG_ENDIAN
+			   "wfb",
+#else
+			   "fb",
+#endif
+			   NULL, NULL, NULL, NULL,
                            NULL, NULL)) {
             return NULL;
         }
@@ -406,13 +430,6 @@ static pointer XGISetup(pointer module,
             return NULL;
         }
 
-        LoaderReqSymLists(vgahwSymbols, fbSymbols, driSymbols, drmSymbols,
-                          NULL);
-
-
-        LoaderRefSymLists(i2cSymbols, ramdacSymbols, 
-                          xaaSymbols, shadowSymbols, fbdevHWSymbols, NULL);
-
         /*
          * The return value must be non-NULL on success even though
          * there is no TearDownProc.
@@ -619,10 +636,10 @@ static Bool XGIProbe(DriverPtr drv, int flags)
 #endif /* XGIDUALVIEW */
 			}
         }
-        xfree(usedChips);
+        free(usedChips);
     }
 
-    xfree(devSections);
+    free(devSections);
     return foundScreen;
 }
 #endif
@@ -655,7 +672,7 @@ static void XGIFreeRec(ScrnInfoPtr pScrn)
     {
         return;
     }
-    xfree(pScrn->driverPrivate);
+    free(pScrn->driverPrivate);
     pScrn->driverPrivate = NULL;
 
 #if DBG_FLOW
@@ -664,36 +681,6 @@ static void XGIFreeRec(ScrnInfoPtr pScrn)
 
 }
 
-/* XGI Display Power Management Set */
-static void XGIDPMSSet(ScrnInfoPtr pScrn, int PowerManagementMode, int flags)
-{
-    XGIPtr  pXGI = XGIPTR(pScrn);
-
-
-    if (!pScrn->vtSema) {
-        return;
-    }
-
-    if (pXGI->isFBDev) {
-        fbdevHWDPMSSet(pScrn, PowerManagementMode, flags);
-    } else if (pXGI->pVbe) {
-	/* I don't know if the bug is in XGI's BIOS or in VBEDPMSSet, but
-	 * cx must be set to 0 here, or the mode will not be set.
-	 */
-        pXGI->pInt10->cx = 0x0000;
-        VBEDPMSSet(pXGI->pVbe, PowerManagementMode);
-    } else {
-        const uint8_t power_status = (IN3CFB(0x23) & ~0x03) 
-	    | (PowerManagementMode);
-        const uint8_t pm_ctrl = (IN3C5B(0x24) & ~0x01)
-            | ((PowerManagementMode == DPMSModeOn) ? 0x01 : 0x00);
-
-
-        OUT3CFB(0x23, power_status);
-        OUT3C5B(0x24, pm_ctrl);
-    }
-}
-
 /**
  * Memory map the MMIO region.  Used during pre-init.
  */
@@ -1023,8 +1010,6 @@ static Bool XGIPreInitInt10(ScrnInfoPtr pScrn)
         return FALSE;
     }
 
-    xf86LoaderReqSymLists(vbeSymbols, int10Symbols, NULL);
-
 #if DBG_FLOW
     xf86DrvMsg(pScrn->scrnIndex, X_INFO, "-- Leave %s() %s:%d\n", __FUNCTION__, __FILE__, __LINE__);
 #endif
@@ -1293,7 +1278,6 @@ static Bool XGIPreInitCursor(ScrnInfoPtr pScrn)
     if (!xf86ReturnOptValBool(pXGI->pOptionInfo, OPTION_SW_CURSOR, FALSE))
     {
         if (!xf86LoadSubModule(pScrn, "ramdac")) return FALSE;
-        xf86LoaderReqSymLists(ramdacSymbols, NULL);
     }
 
 #if DBG_FLOW
@@ -1320,8 +1304,6 @@ static Bool XGIPreInitAccel(ScrnInfoPtr pScrn)
          */
         if (!g_DualViewMode || pXGI->FirstView) 
             if (!xf86LoadSubModule(pScrn, "xaa")) return FALSE;
-
-        xf86LoaderReqSymLists(xaaSymbols, NULL);
     }
 
 #if DBG_FLOW
@@ -1482,7 +1464,6 @@ static Bool XGIPreInitShadow(ScrnInfoPtr pScrn)
             XGIFreeRec(pScrn);
             return FALSE;
         }
-        xf86LoaderReqSymLists(shadowSymbols, NULL);
     }
 
 #if DBG_FLOW
@@ -1499,7 +1480,6 @@ xf86MonPtr get_configured_monitor(ScrnInfoPtr pScrn, int index)
 
 
     if (xf86LoadSubModule(pScrn, "i2c")) {
-        xf86LoaderReqSymLists(i2cSymbols, NULL);
 	if (!xg47_InitI2C(pScrn)) {
 	    xf86DrvMsg(pScrn->scrnIndex, X_ERROR,
 		       "I2C initialization failed!\n");
@@ -1514,15 +1494,12 @@ xf86MonPtr get_configured_monitor(ScrnInfoPtr pScrn, int index)
         return NULL;
     }
 
-    xf86LoaderReqSymLists(ddcSymbols, NULL);
-
-
     if (pXGI->pI2C != NULL) {
-        pMon = xf86DoEDID_DDC2(pScrn->scrnIndex, pXGI->pI2C);
+	pMon = xf86DoEDID_DDC2(XF86_SCRN_ARG(pScrn), pXGI->pI2C);
     }
 
     if (pMon == NULL) {
-        pMon = xf86DoEDID_DDC1(pScrn->scrnIndex, vgaHWddc1SetSpeedWeak(),
+        pMon = xf86DoEDID_DDC1(XF86_SCRN_ARG(pScrn), vgaHWddc1SetSpeedWeak(),
                                XG47DDCRead);
     }
 
@@ -1567,7 +1544,9 @@ Bool XGIPreInit(ScrnInfoPtr pScrn, int flags)
     /* Get the entity, and make sure it is PCI. */
     pXGI->pEnt = xf86GetEntityInfo(pScrn->entityList[0]);
 
+#ifndef XSERVER_LIBPCIACCESS
     if (pXGI->pEnt->resources) return FALSE;
+#endif
     if (pXGI->pEnt->location.type != BUS_PCI)   goto fail;
 
 /* Jong 09/06/2006; support dual view */
@@ -1613,6 +1592,7 @@ Bool XGIPreInit(ScrnInfoPtr pScrn, int flags)
         XGIFreeRec(pScrn);
         return FALSE;
     }
+    vgaHWSetStdFuncs(VGAHWPTR(pScrn));
 
     /* All additional resources that the screen needs must be registered here.
      * This should be done with xf86RegisterResources(). If some of the fixed resources
@@ -1621,13 +1601,14 @@ Bool XGIPreInit(ScrnInfoPtr pScrn, int flags)
      * xf86SetOperatingState().
      */
 
+#ifndef XSERVER_LIBPCIACCESS
     /* Register the PCI-assigned resources. */
     if (xf86RegisterResources(pXGI->pEnt->index, NULL, ResExclusive))
     {
         xf86DrvMsg(pScrn->scrnIndex, X_ERROR, "xf86RegisterResources() found resource conflicts\n");
         return FALSE;
     }
-
+#endif
     /*
      * Set the depth/bpp.  Use the globally preferred depth/bpp.  If the driver
      * has special default depth/bpp requirements, the defaults should be specified
@@ -1653,7 +1634,6 @@ Bool XGIPreInit(ScrnInfoPtr pScrn, int flags)
 
 	/* Jong 07/07/2006; has error - _dl_catch_error() from /lib/ld-linux.so.2 ???? */
         if (!xf86LoadSubModule(pScrn, "fbdevhw")) return FALSE;
-        xf86LoaderReqSymLists(fbdevHWSymbols, NULL);
 
         /* check for linux framebuffer device */
         if (!fbdevHWInit(pScrn, pXGI->pPciInfo, NULL)) return FALSE;
@@ -1708,14 +1688,6 @@ Bool XGIPreInit(ScrnInfoPtr pScrn, int flags)
     }
 
 
-    /* Free the video bios (if applicable) */
-    if (pXGI->biosBase)
-    {
-        xfree(pXGI->biosBase);
-        pXGI->biosBase = NULL;
-    }
-
-
     /* Decide which operations need to be placed under resource access control.
      * The classes of operations are the frame buffer operations (RAC_FB),
      * the pointer operations (RAC_CURSOR), the viewport change operations (RAC_VIEWPORT)
@@ -1724,7 +1696,7 @@ Bool XGIPreInit(ScrnInfoPtr pScrn, int flags)
      * This can be specified separately for memory and IO resources (the racMemFlags
      * and racIoFlags fields of the ScrnInfoRec respectively).
      */
-
+#ifndef XSERVER_LIBPCIACCESS
     pScrn->racMemFlags = RAC_FB | RAC_COLORMAP | RAC_CURSOR | RAC_VIEWPORT;
 
     if (pXGI->isMmioOnly)
@@ -1735,7 +1707,7 @@ Bool XGIPreInit(ScrnInfoPtr pScrn, int flags)
     {
         pScrn->racIoFlags = RAC_FB | RAC_COLORMAP | RAC_CURSOR | RAC_VIEWPORT;
     }
-
+#endif
 /* Jong 09/11/2006; support dual view */
 #ifdef XGIDUALVIEW
     xf86SetPrimInitDone(pScrn->entityList[0]);
@@ -1744,12 +1716,6 @@ Bool XGIPreInit(ScrnInfoPtr pScrn, int flags)
     return TRUE;
 fail:
     XGIUnmapMem(pScrn);
-    /* Free the video bios (if applicable) */
-    if (pXGI->biosBase)
-    {
-      xfree(pXGI->biosBase);
-      pXGI->biosBase = NULL;
-    }
 
     if (pXGI->pVbe) {
         vbeFree(pXGI->pVbe);
@@ -1773,10 +1739,10 @@ fail:
     return FALSE;
 }
 
-static void XGIBlockHandler(int i, pointer pBlockData, pointer pTimeout, pointer pReadmask)
+static void XGIBlockHandler(BLOCKHANDLER_ARGS_DECL)
 {
-    ScreenPtr      pScreen = screenInfo.screens[i];
-    ScrnInfoPtr    pScrn = xf86Screens[i];
+    SCREEN_PTR(arg);
+    ScrnInfoPtr    pScrn = xf86ScreenToScrn(pScreen);
     XGIPtr         pXGI = XGIPTR(pScrn);
 
 #if DBG_FLOW
@@ -1784,7 +1750,7 @@ static void XGIBlockHandler(int i, pointer pBlockData, pointer pTimeout, pointer
 #endif
 
     pScreen->BlockHandler = pXGI->BlockHandler;
-    (*pScreen->BlockHandler)(i, pBlockData, pTimeout, pReadmask);
+    (*pScreen->BlockHandler)(BLOCKHANDLER_ARGS);
     pScreen->BlockHandler = XGIBlockHandler;
 
 #ifdef XvExtension
@@ -1845,12 +1811,14 @@ static void XGIRestore(ScrnInfoPtr pScrn)
     vgaRegPtr   pVgaReg = &VGAHWPTR(pScrn)->SavedReg;
 
     xg47_mode_restore(pScrn, pVgaReg, pXGIReg);
+#if !defined(__powerpc__)
     vgaHWRestore(pScrn, pVgaReg, VGA_SR_FONTS);
+#endif
 }
 
 Bool XGIFBManagerInit(ScreenPtr pScreen)
 {
-    ScrnInfoPtr     pScrn = xf86Screens[pScreen->myNum];
+    ScrnInfoPtr     pScrn = xf86ScreenToScrn(pScreen);
     XGIPtr          pXGI = XGIPTR(pScrn);
     BoxRec          availFBArea;
     CARD16          temp;
@@ -1880,10 +1848,10 @@ Bool XGIFBManagerInit(ScreenPtr pScreen)
 }
 
 /* Called at the start of each server generation. */
-Bool XGIScreenInit(int scrnIndex, ScreenPtr pScreen, int argc, char **argv)
+Bool XGIScreenInit(SCREEN_INIT_ARGS_DECL)
 {
     /* First Get the ScrnInfoRec */
-    ScrnInfoPtr pScrn  = xf86Screens[pScreen->myNum];
+    ScrnInfoPtr pScrn  = xf86ScreenToScrn(pScreen);
     XGIPtr      pXGI = XGIPTR(pScrn);
     vgaHWPtr    pVgaHW = VGAHWPTR(pScrn);
     VisualPtr   pVisual;
@@ -1950,7 +1918,7 @@ Bool XGIScreenInit(int scrnIndex, ScreenPtr pScreen, int argc, char **argv)
     /* Darken the screen for aesthetic reasons and set the viewport.
      */
     XGISaveScreen(pScreen, SCREEN_SAVER_ON);
-    pScrn->AdjustFrame(scrnIndex, pScrn->frameX0, pScrn->frameY0, 0);
+    pScrn->AdjustFrame(ADJUST_FRAME_ARGS(pScrn, pScrn->frameX0, pScrn->frameY0));
 
 
     /*
@@ -2008,7 +1976,7 @@ Bool XGIScreenInit(int scrnIndex, ScreenPtr pScreen, int argc, char **argv)
     if (pXGI->isShadowFB)
     {
         pXGI->shadowPitch = BitmapBytePad(pScrn->bitsPerPixel * width);
-        pXGI->pShadow = xalloc(pXGI->shadowPitch * height);
+        pXGI->pShadow = malloc(pXGI->shadowPitch * height);
         displayWidth = pXGI->shadowPitch / (pScrn->bitsPerPixel >> 3);
         pFBStart = pXGI->pShadow;
     }
@@ -2027,11 +1995,19 @@ Bool XGIScreenInit(int scrnIndex, ScreenPtr pScreen, int argc, char **argv)
 pScrn->xDpi = 72;
 pScrn->yDpi = 72;
 pScrn->pScreen = pScreen;
+#if X_BYTE_ORDER == X_BIG_ENDIAN
+    retValue = wfbScreenInit(pScreen, pFBStart, width, height,
+			     pScrn->xDpi, pScrn->yDpi, displayWidth,
+			     pScrn->bitsPerPixel,
+			     xg47_setup_fb_wrap, xg47_finish_fb_wrap);
+#else
     retValue = fbScreenInit(pScreen, pFBStart, width, height,
-                            pScrn->xDpi, pScrn->yDpi, displayWidth,
-                            pScrn->bitsPerPixel);
+			    pScrn->xDpi, pScrn->yDpi, displayWidth,
+			    pScrn->bitsPerPixel);
+#endif
 
     if (!retValue)  goto fail;
+
     /*
      * Set initial black & white colourmap indices.
      */
@@ -2054,11 +2030,15 @@ pScrn->pScreen = pScreen;
     }
 
     /* must be after RGB order fixed */
+#if X_BYTE_ORDER == X_BIG_ENDIAN
+    wfbPictureInit(pScreen, 0, 0);
+#else
     fbPictureInit(pScreen, 0, 0);
+#endif
 
     if (!XGIFBManagerInit(pScreen))
     {
-        xf86DrvMsg(scrnIndex, X_ERROR, "FB Manager init failed \n");
+        xf86DrvMsg(pScrn->scrnIndex, X_ERROR, "FB Manager init failed \n");
     }
 
     /* If backing store is to be supported (as is usually the case), initialise it. */
@@ -2094,18 +2074,18 @@ pScrn->pScreen = pScreen;
     if (!pXGI->noAccel) {
         pXGI->noAccel = !XG47AccelInit(pScreen);
         if (pXGI->noAccel) {
-            xf86DrvMsg(scrnIndex, X_ERROR, "Acceleration initialization failed\n");
+            xf86DrvMsg(pScrn->scrnIndex, X_ERROR, "Acceleration initialization failed\n");
         }
     }
 
     if (!pXGI->noAccel) {
-        xf86DrvMsg(scrnIndex, X_INFO, "Acceleration enabled\n");
+        xf86DrvMsg(pScrn->scrnIndex, X_INFO, "Acceleration enabled\n");
     } else {
-        xf86DrvMsg(scrnIndex, X_INFO, "Acceleration disabled\n");
+        xf86DrvMsg(pScrn->scrnIndex, X_INFO, "Acceleration disabled\n");
     }
 
 
-    if (!XGIEnterVT(scrnIndex, 0))
+    if (!XGIEnterVT(VT_FUNC_ARGS))
 	goto fail;
 
     /* Set Silken Mouse */
@@ -2128,12 +2108,12 @@ pScrn->pScreen = pScreen;
                        (int) pXGI->cursorStart / (int) pScrn->displayWidth);
             if (xf86QueryLargestOffscreenArea(pScreen, &width, &height,
                                               0, 0, 0)) {
-                xf86DrvMsg(scrnIndex, X_INFO,
+                xf86DrvMsg(pScrn->scrnIndex, X_INFO,
                            "Largest offscreen area available: %d x %d\n",
                            width, height);
             }
         } else {
-            xf86DrvMsg(scrnIndex, X_ERROR,
+            xf86DrvMsg(pScrn->scrnIndex, X_ERROR,
                        "Hardware cursor initialization failed\n");
             pXGI->isHWCursor = FALSE;
         }
@@ -2141,7 +2121,7 @@ pScrn->pScreen = pScreen;
 
     if (!pXGI->isHWCursor) {
         pXGI->cursorStart = 0;
-        xf86DrvMsg(scrnIndex, X_INFO, "Using software cursor\n");
+        xf86DrvMsg(pScrn->scrnIndex, X_INFO, "Using software cursor\n");
     }
 
 
@@ -2238,14 +2218,7 @@ fail:
     PDEBUG(ErrorF("Jong-After-XGIInitMC-7\n"));
     XGIUnmapMem(pScrn);
     PDEBUG(ErrorF("Jong-After-XGIInitMC-8\n"));
-    /* Free the video bios (if applicable) */
-    if (pXGI->biosBase)
-    {
-        xfree(pXGI->biosBase);
-        pXGI->biosBase = NULL;
-    }
 
-    PDEBUG(ErrorF("Jong-After-XGIInitMC-9\n"));
     if (pXGI->pVbe) {
         vbeFree(pXGI->pVbe);
         pXGI->pVbe = NULL;
@@ -2273,9 +2246,10 @@ fail:
  * Initialises the new mode for the screen identified by index.
  * The viewport may need to be adjusted also.
  */
-Bool XGISwitchMode(int scrnIndex, DisplayModePtr mode, int flags)
+Bool XGISwitchMode(SWITCH_MODE_ARGS_DECL)
 {
-    return XGIModeInit(xf86Screens[scrnIndex], mode);
+    SCRN_INFO_PTR(arg);
+    return XGIModeInit(pScrn, mode);
 }
 
 
@@ -2283,9 +2257,9 @@ Bool XGISwitchMode(int scrnIndex, DisplayModePtr mode, int flags)
  * This is called when VT switching back to the X server. Its job is
  * to reinitialise the video mode.
  */
-static Bool XGIEnterVT(int scrnIndex, int flags)
+static Bool XGIEnterVT(VT_FUNC_ARGS_DECL)
 {
-    ScrnInfoPtr pScrn = xf86Screens[scrnIndex];
+    SCRN_INFO_PTR(arg);
     XGIPtr      pXGI = XGIPTR(pScrn);
 
 #if DBG_FLOW
@@ -2308,7 +2282,7 @@ static Bool XGIEnterVT(int scrnIndex, int flags)
     }
 
     if (pXGI->isFBDev) {
-        if (!fbdevHWEnterVT(scrnIndex,flags))
+        if (!fbdevHWEnterVT(VT_FUNC_ARGS))
             return FALSE;
     } else {
         /* Should we re-save the text mode on each VT enter? */
@@ -2321,7 +2295,7 @@ static Bool XGIEnterVT(int scrnIndex, int flags)
         xg47_Reset(pXGI->cmdList);
     }
 
-    pScrn->AdjustFrame(scrnIndex, pScrn->frameX0, pScrn->frameY0, 0);
+    pScrn->AdjustFrame(ADJUST_FRAME_ARGS(pScrn, pScrn->frameX0, pScrn->frameY0));
 
 #if DBG_FLOW
     xf86DrvMsg(pScrn->scrnIndex, X_INFO, "-- Leave %s() %s:%d\n", __FUNCTION__, __FILE__, __LINE__);
@@ -2338,9 +2312,9 @@ static Bool XGIEnterVT(int scrnIndex, int flags)
  */
 
 /* Mandatory */
-static void XGILeaveVT(int scrnIndex, int flags)
+static void XGILeaveVT(VT_FUNC_ARGS_DECL)
 {
-    ScrnInfoPtr pScrn = xf86Screens[scrnIndex];
+    SCRN_INFO_PTR(arg);
     XGIPtr      pXGI = XGIPTR(pScrn);
 
 #if DBG_FLOW
@@ -2365,7 +2339,7 @@ static void XGILeaveVT(int scrnIndex, int flags)
     }
 
     if (pXGI->isFBDev) {
-        fbdevHWLeaveVT(scrnIndex,flags);
+        fbdevHWLeaveVT(VT_FUNC_ARGS);
     } else {
         XGIRestore(pScrn);
     }
@@ -2398,9 +2372,9 @@ static void XGILeaveVT(int scrnIndex, int flags)
  */
 
 /* Mandatory */
-static Bool XGICloseScreen(int scrnIndex, ScreenPtr pScreen)
+static Bool XGICloseScreen(CLOSE_SCREEN_ARGS_DECL)
 {
-    ScrnInfoPtr pScrn = xf86Screens[scrnIndex];
+    ScrnInfoPtr pScrn = xf86ScreenToScrn(pScreen);
     vgaHWPtr    pVgaHW = VGAHWPTR(pScrn);
     XGIPtr      pXGI = XGIPTR(pScrn);
     Bool        result;
@@ -2418,12 +2392,15 @@ static Bool XGICloseScreen(int scrnIndex, ScreenPtr pScreen)
     /* Do only one time; otherwise will cause system hang 
      */
     if ((g_DualViewMode == 0 ) || (pScreen->myNum == 1)) {
+#ifdef HAVE_XAA_H
 	if (pXGI->pXaaInfo) {
 	    XG47AccelExit(pScreen);
 	    XAADestroyInfoRec(pXGI->pXaaInfo);
 
 	    pXGI->pXaaInfo = NULL;
 	}
+#endif
+
     }
 
     /* Restore the saved video state and unmap the memory regions. */
@@ -2444,19 +2421,19 @@ static Bool XGICloseScreen(int scrnIndex, ScreenPtr pScreen)
 
     if (pXGI->pShadow)
     {
-        xfree(pXGI->pShadow);
+        free(pXGI->pShadow);
         pXGI->pShadow = NULL;
     }
 
     if (pXGI->pDgaModes)
     {
-        xfree(pXGI->pDgaModes);
+        free(pXGI->pDgaModes);
         pXGI->pDgaModes = NULL;
     }
 
     if (pXGI->pAdaptor)
     {
-        /* xfree(pXGI->pAdaptor->pPortPrivates[0].ptr); */
+        /* free(pXGI->pAdaptor->pPortPrivates[0].ptr); */
         xf86XVFreeVideoAdaptorRec(pXGI->pAdaptor);
         pXGI->pAdaptor = NULL;
     }
@@ -2486,7 +2463,7 @@ static Bool XGICloseScreen(int scrnIndex, ScreenPtr pScreen)
      */
     pScreen->CloseScreen = pXGI->CloseScreen;
 
-    result = (*pScreen->CloseScreen)(scrnIndex, pScreen);
+    result = (*pScreen->CloseScreen)(CLOSE_SCREEN_ARGS);
 
 #if DBG_FLOW
     xf86DrvMsg(pScrn->scrnIndex, X_INFO, "-- Leave %s() %s:%d\n", __FUNCTION__, __FILE__, __LINE__);
@@ -2497,7 +2474,7 @@ static Bool XGICloseScreen(int scrnIndex, ScreenPtr pScreen)
 
 static Bool XGISaveScreen(ScreenPtr pScreen, int mode)
 {
-    ScrnInfoPtr pScrn = xf86Screens[pScreen->myNum];
+    ScrnInfoPtr pScrn = xf86ScreenToScrn(pScreen);
     Bool        unblank;
 
 #if DBG_FLOW
@@ -2528,9 +2505,9 @@ static Bool XGISaveScreen(ScreenPtr pScreen, int mode)
  * It would include the driverPrivate, and any ``privates'' entries that modules
  * may have allocated.
  */
-static void XGIFreeScreen(int scrnIndex, int flags)
+static void XGIFreeScreen(FREE_SCREEN_ARGS_DECL)
 {
-    ScrnInfoPtr pScrn = xf86Screens[scrnIndex];
+    SCRN_INFO_PTR(arg);
 
 #if DBG_FLOW
     xf86DrvMsg(pScrn->scrnIndex, X_INFO, "++ Enter %s() %s:%d\n", __FUNCTION__, __FILE__, __LINE__);
@@ -2569,9 +2546,9 @@ static void XGIFreeScreen(int scrnIndex, int flags)
  * depend on any mode other than the one being validated, while calls with
  * MODECHECK_FINAL are intended for checks that may involve more than one mode.
  */
-static int XGIValidMode(int scrnIndex, DisplayModePtr mode, Bool verbose, int flags)
+static int XGIValidMode(SCRN_ARG_TYPE arg, DisplayModePtr mode, Bool verbose, int flags)
 {
-    ScrnInfoPtr     pScrn = xf86Screens[scrnIndex];
+    SCRN_INFO_PTR(arg);
     XGIPtr          pXGI = XGIPTR(pScrn);
     int             ret;
 
@@ -2603,3 +2580,139 @@ xg47_crtc_config_resize(ScrnInfoPtr scrn, int width, int height)
  
     return TRUE;
 }
+
+
+#if X_BYTE_ORDER == X_BIG_ENDIAN
+uint32_t
+xg47_read_memory_swap_func(const void *src, int size)
+{
+    switch (size) {
+    case 1: {
+	const uint8_t *const ptr = src;
+	return *ptr;
+    }
+
+    case 2: {
+	const uint16_t *const ptr = src;
+	return bswap_16(*ptr);
+    }
+
+    case 4: {
+	const uint32_t *const ptr = src;
+	return bswap_32(*ptr);
+    }
+    }
+
+    return 0;
+}
+
+
+void
+xg47_write_memory_swap_func(void *dst, uint32_t value, int size)
+{
+    switch (size) {
+    case 1: {
+	uintptr_t addr = (uintptr_t) dst;
+	uint8_t *const ptr = (uint8_t *)(addr ^ 3);
+
+	*ptr = value;
+	break;
+    }
+
+    case 2: {
+	uint16_t *const ptr = dst;
+	*ptr = bswap_16(value);
+	break;
+    }
+
+    case 4: {
+	uint32_t *const ptr = dst;
+	*ptr = bswap_32(value);
+	break;
+    }
+    }
+}
+
+
+uint32_t
+xg47_read_memory_func(const void *src, int size)
+{
+    switch (size) {
+    case 1: {
+#if 0
+	uintptr_t addr = (uintptr_t) src;
+	const uint8_t *const ptr = (uint8_t *)(addr ^ 3);
+#else
+	const uint8_t *const ptr = (const uint8_t *) src;
+#endif
+	return *ptr;
+    }
+
+    case 2: {
+	const uint16_t *const ptr = src;
+	return *ptr;
+    }
+
+    case 4: {
+	const uint32_t *const ptr = src;
+	return *ptr;
+    }
+    }
+
+    return 0;
+}
+
+
+void
+xg47_write_memory_func(void *dst, uint32_t value, int size)
+{
+    switch (size) {
+    case 1: {
+	uint8_t *const ptr = dst;
+	*ptr = value;
+	break;
+    }
+
+    case 2: {
+	uint16_t *const ptr = dst;
+	*ptr = value;
+	break;
+    }
+
+    case 4: {
+	uint32_t *const ptr = dst;
+	*ptr = value;
+	break;
+    }
+    }
+}
+
+
+void
+xg47_setup_fb_wrap(ReadMemoryProcPtr *read_ptr, 
+		   WriteMemoryProcPtr *write_ptr,
+		   DrawablePtr pDraw)
+{
+    switch (pDraw->type) {
+    case DRAWABLE_WINDOW:
+#ifdef DRAWABLE_BUFFER
+    case DRAWABLE_BUFFER:
+#endif
+	*read_ptr = xg47_read_memory_swap_func;
+	*write_ptr = xg47_write_memory_swap_func;
+	break;
+	
+    default:
+	*read_ptr = xg47_read_memory_func;
+	*write_ptr = xg47_write_memory_func;
+	break;
+    }
+}
+
+
+void
+xg47_finish_fb_wrap(DrawablePtr pDraw)
+{
+    (void) pDraw;
+}
+#endif /* X_BYTE_ORDER == X_BIG_ENDIAN */
